--                                                                    --
--  package Gtk.Missed              Copyright (c)  Maxim Reznik       --
--  Interface                                      Summer, 2006       --
--                                                                    --
--                                Last revision :  19:03 27 May 2009  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Gdk.Rectangle;         use Gdk.Rectangle;
with Gdk.Types;             use Gdk.Types;
with Glib.Values;           use Glib.Values;
with Gtk.Container;         use Gtk.Container;
with Gtk.Tree_Model;        use Gtk.Tree_Model;
with Gtk.Tree_Store;        use Gtk.Tree_Store;
with Gtk.Tree_View;         use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with Gtk.Widget;            use Gtk.Widget;
with Gtk.Window;            use Gtk.Window;

with Glib;
with Glib.Object;
with Glib.Properties;
with Glib.Wrappers;
with Gtk.RC;

package Gtk.Missed is
   --
   -- GtkAda_Contributions_Domain -- The domain name used for the  error
   --                                log messages  coming  out  of  this
   --                                library.
   --
   GtkAda_Contributions_Domain : constant String := "GtkAda+";

   type Row_Order is (Before, Equal, After);

   type User_Directory is               -- The user's
        (  User_Directory_Desktop,      -- Desktop directory
           User_Directory_Documents,    -- Documents directory
           User_Directory_Download,     -- Downloads directory
           User_Directory_Music,        -- Music directory
           User_Directory_Pictures,     -- Pictures directory
           User_Directory_Public_Share, -- shared directory
           User_Directory_Templates,    -- Templates directory
           User_Directory_Videos        -- Movies directory
        );

   type GFileTest is new GUInt;
   File_Test_Is_Regular    : constant GFileTest := 2**0;
   File_Test_Is_Symlink    : constant GFileTest := 2**1;
   File_Test_Is_Dir        : constant GFileTest := 2**2;
   File_Test_Is_Executable : constant GFileTest := 2**3;
   File_Test_Exists        : constant GFileTest := 2**4;

   procedure Add_Class_Style
             (  Style   : Gtk.RC.Gtk_RC_Style;
                Pattern : UTF8_String
             );

   procedure Add_Widget_Class_Style
             (  Style   : Gtk.RC.Gtk_RC_Style;
                Pattern : UTF8_String
             );

   procedure Add_Widget_Name_Style
             (  Style   : Gtk.RC.Gtk_RC_Style;
                Pattern : UTF8_String
             );
--
-- Build_Filename -- File base name
--
--    First_Element  - The prefix of a file name
--    Second_Element - The suffix of
--    Third_Element  - The suffix of the suffix
--    Fourth_Element - ...
--    Fifth_Element  - ...
--
-- This  function  creates  a  filename  from  two of elements using the
-- correct  separator  for  filenames.  On  Unix,  this function behaves
-- identically to g_build_path (G_DIR_SEPARATOR_S, first_element, ....).
-- On Windows, it takes into account that either  the  backslash  (\  or
-- slash (/) can be  used  as  separator  in  filenames,  but  otherwise
-- behaves as  on  Unix.  When  file  pathname  separators  need  to  be
-- inserted, the one that last previously  occurred  in  the  parameters
-- (reading from left to right) is used. No attempt is made to force the
-- resulting filename to be an absolute path. If the first element is  a
-- relative path, the result will be a relative path.
--
-- Returns :
--
--    The file name
--
   function Build_Filename
            (  First_Element  : UTF8_String;
               Second_Element : UTF8_String
            )  return UTF8_String;
   function Build_Filename
            (  First_Element  : UTF8_String;
               Second_Element : UTF8_String;
               Third_Element  : UTF8_String
            )  return UTF8_String;
   function Build_Filename
            (  First_Element  : UTF8_String;
               Second_Element : UTF8_String;
               Third_Element  : UTF8_String;
               Fourth_Element : UTF8_String
            )  return UTF8_String;
   function Build_Filename
            (  First_Element  : UTF8_String;
               Second_Element : UTF8_String;
               Third_Element  : UTF8_String;
               Fourth_Element : UTF8_String;
               Fifth_Element  : UTF8_String
            )  return UTF8_String;

   function Class_From_Type (Typ : GType) return GObject_Class;

   function Compare
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Path
            )  return Row_Order;

   function Compare
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Path
            )  return Row_Order;

   function Compare
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Iter
            )  return Row_Order;

   function Compare
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Iter
            )  return Row_Order;
--
-- Erase -- Elements of the container
--
--    Container - The container
--
   procedure Erase (Container : access Gtk_Container_Record'Class);
--
-- File_Test -- Test file status
--
--    File_Name - The file name
--    Flags     - To test for
--
-- Returns :
--
--    True if any of the Flags is set
--
   function File_Test
            (  File_Name : UTF8_String;
               Flags     : GFileTest
            )  return Boolean;
--
-- Find_Program_In_Path -- Find an executable program by name
--
--    Program - The program name
--
-- This function locates the  first  executable  named  Program  in  the
-- user's path.
--
-- Returns :
--
--    Absolute path of the program or else empty string
--
   function Find_Program_In_Path (Program : UTF8_String)
      return UTF8_String;
--
-- Is_In -- Elements of the container
--
--    Container - The container
--    Element   - Of the container
--
-- Returns :
--
--    True if the
--
   function Is_In
            (  Container : access Gtk_Container_Record'Class;
               Element   : access Gtk_Widget_Record'Class
            )  return Boolean;
--
-- Get_Application_Name -- Returns application name
--
-- Returns :
--
--    The application name in a human readable form
--
   function Get_Application_Name return UTF8_String;
--
-- Get_Current_Dir -- Returns current directory
--
-- Returns :
--
--    Current directory
--
   function Get_Current_Dir return UTF8_String;
--
-- Get_PRGName -- Returns program name
--
-- Returns :
--
--    The program name
--
   function Get_PRGName return UTF8_String;

   function Is_A (Derived, Ancestor : GType) return Boolean;

   function Is_In
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Path
            )  return Boolean;

   function Is_In
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Path
            )  return Boolean;

   function Is_In
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Iter
            )  return Boolean;

   function Is_In
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Iter
            )  return Boolean;

   function Is_Parent
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Path
            )  return Boolean;

   function Is_Parent
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Path
            )  return Boolean;

   function Is_Parent
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Iter
            )  return Boolean;

   function Is_Parent
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Iter
            )  return Boolean;

   function Is_Sibling
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Path
            )  return Boolean;

   function Is_Sibling
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Path
            )  return Boolean;

   function Is_Sibling
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Path;
               B     : Gtk_Tree_Iter
            )  return Boolean;

   function Is_Sibling
            (  Model : access Gtk_Tree_Model_Record'Class;
               A     : Gtk_Tree_Iter;
               B     : Gtk_Tree_Iter
            )  return Boolean;

   function Get_Background_Area
            (  Tree_View : access Gtk_Tree_View_Record'Class;
               Path      : Gtk_Tree_Path;
               Column    : Gtk_Tree_View_Column := null
            )  return Gdk_Rectangle;
--
-- Get_Basename -- File base name
--
--    File_Name - A file name
--
-- Gets the last component of the filename. If  file_name  ends  with  a
-- directory  separator  it gets the component before the last slash. If
-- file_name consists only of  directory  separators  (and  on  Windows,
-- possibly  a  drive  letter),  a  single  separator  is  returned.  If
-- file_name is empty, it gets ".".
--
-- Returns :
--
--    The base name
--
   function Get_Basename (File_Name : UTF8_String) return UTF8_String;
--
-- Get_Column -- Get column from a value
--
--   Value - A column value
--
-- This function can be  used  within  a  handler  to  obtain  a  column
-- parameter  from  GLib  value. The list of parameters is passed to the
-- handler as GValues. The function Nth is may  be  used  to  extract  a
-- parameter. The result is GValue to which Get_Column is then applied.
--
-- Returns :
--
--    The column
--
   function Get_Column (Value : GValue) return Gtk_Tree_View_Column;
--
-- Get_Column_No -- Get column number
--
--   Tree_View - A tree view widget
--   Column    - A column of
--
-- Returns :
--
--    The number of Column 0.. or else -1
--
   function Get_Column_No
            (  Tree_View : access Gtk_Tree_View_Record'Class;
               Column    : access Gtk_Tree_View_Column_Record'Class
            )  return GInt;
--
-- Get_Dirname -- File directory name
--
--    File_Name - A file name
--
-- Gets the directory components of a file name. If the file name has no
-- directory components "." is returned.
--
-- Returns :
--
--    The directory name
--
   function Get_Dirname (File_Name : UTF8_String) return UTF8_String;
--
-- Get_Root -- Of a file name
--
--    File_Name - A file name
--
-- This function returns the root component,of a name, i.e. the  "/"  in
-- UNIX or "C:\" under Windows.
--
-- Returns :
--
--    The file name's root component
--
-- Exceptions :
--
--    Use_Error - File_Name is not an absolute name
--
   function Get_Root (File_Name : UTF8_String) return UTF8_String;

   procedure Get_Size
             (  Window : access Gtk_Window_Record'Class;
                Width  : out Glib.Gint;
                Height : out Glib.Gint
             );
--
-- Get_Row_No -- Get row number
--
--   Tree_View   - A tree view model
--   Iter / Path - An iterator or path in the model
--
-- Returns :
--
--    The row number 0.. or else -1
--
   function Get_Row_No
            (  Model : access Gtk_Tree_Model_Record'Class;
               Path  : Gtk_Tree_Path
            )  return GInt;
   function Get_Row_No
            (  Model : access Gtk_Tree_Model_Record'Class;
               Iter  : Gtk_Tree_Iter
            )  return GInt;
--
-- Get_User_Special_Dir -- Directories associated with the user
--
--    Directory - The directory
--
-- Returns :
--
--    The directory requested
--
   function Get_User_Special_Dir (Directory : User_Directory)
      return UTF8_String;

   procedure Get_Visible_Range
             (  Tree_View  : access Gtk_Tree_View_Record'Class;
                Start_Path : out Gtk_Tree_Path;
                End_Path   : out Gtk_Tree_Path
             );
--
-- Is_Absolute -- File path check
--
--    File_Name - A file name
--
-- The  function returns true if the given File_Name is an absolute file
-- name, i.e. it contains a full path from the root  directory  such  as
-- "/usr/local" on UNIX or "C:\windows" on Windows systems.
--
-- Returns :
--
--    True if the name is absolute
--
   function Is_Absolute (File_Name : UTF8_String) return Boolean;

   function Keyval_To_Unicode (Key_Val : Gdk_Key_Type) return GUnichar;
--
-- Keyval_To_UTF8 -- Key to UTF8 conversion
--
--    Key_Val - A key value associated with an event
--
-- When Key_Val does not have a corresponding Unicode  character  it  is
-- treated as NUL (code position 0),
--
-- Retunrs :
--
--    The value UTF-8 encoded
--
   function Keyval_To_UTF8 (Key_Val : Gdk_Key_Type) return UTF8_String;

   procedure Set_Property
             (  Object : access Glib.Object.GObject_Record'Class;
                Name   : Glib.Properties.Property_Float;
                Value  : Glib.Gfloat
             );
--
-- Skip_Root -- In a file name
--
--    File_Name - A file name
--
-- This function returns a File_Name part after the root component, i.e.
-- after the "/" in UNIX or "C:\" under Windows.
--
-- Returns :
--
--    The file name without root component
--
-- Exceptions :
--
--    Use_Error - File_Name is not an absolute name
--
   function Skip_Root (File_Name : UTF8_String) return UTF8_String;

   function Unicode_To_Keyval (WC : GUnichar) return Gdk_Key_Type;

private
   pragma Inline (Is_A);
   pragma Import (C, Class_From_Type,   "gtk_type_class");
   pragma Import (C, Keyval_To_Unicode, "gdk_keyval_to_unicode");
   pragma Import (C, Unicode_To_Keyval, "gdk_unicode_to_keyval");
--
-- Search_Data -- To enumerate elements of a container
--
   type Search_Data is record
      Item  : Gtk_Widget;
      Found : Boolean;
   end record;
   type Search_Data_Ptr is access all Search_Data;

   package For_Test is new For_Pkg (Search_Data_Ptr);
   procedure Test
             (  Item : access Gtk_Widget_Record'Class;
                Data : in out Search_Data_Ptr
             );
end Gtk.Missed;
